/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.type;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.xml.parsers.ParserConfigurationException;
import nl.cloudfarming.client.isobus.model.BinaryReader;
import nl.cloudfarming.client.isobus.model.TreatmentZone;
import org.openide.filesystems.FileObject;
import org.xml.sax.SAXException;

/**
 * Class which reads binary grid files
 * There are two types of files
 * The first Grid.type == 1
 * the binary file is a lookup table which contains the Grid.treatmentZoneCode for every square in the grid.
 * 
 * the Second grid.type == 2
 * the binary file contains the values of the ProcessDataVariable.processDataValue this is not yet supported
 * TODO: support type two
 *
 * @author Merijn Zengers
 */
public class GridReader {

    public GridReader() {
    }
    
    /**
     * This method is only intended for binary files of Grid.type == 1
     * Reads the Grid.treatmentZoneCode from the binary file
     * Stores the values in the returned list
     * Will find the treatmentZone in the list with available treatmentZones
     * If found will add to the return list
     * else throws an assertion error
     * Take Notice!!: the values in the file are ordered from the right bottom to left top of the grid 
     * @return a list of the read values
     * @throws IOException 
     */
    public List<TreatmentZone> getTreatMentZones(FileObject gridFile, List<TreatmentZone> availableTreatmentZones) throws IOException,  ParserConfigurationException, SAXException, FileNotFoundException{
        BinaryReader binaryReader = new BinaryReader(gridFile.getInputStream());
        ArrayList<TreatmentZone> values = new ArrayList<>();
        while (binaryReader.available()) {
            long value = binaryReader.read(GridFieldEnum.TREATMENT_ZONE_CODE_TYPE);
            boolean found = false;
            for(TreatmentZone treatmentZone : availableTreatmentZones){
                if(treatmentZone.getTreatmentZoneCode() == value){
                    values.add(treatmentZone);
                    found = true;
                }
            }
            assert(found);            
        }
        return values;
    }
    
    
}
