/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.type;

import java.util.List;
import nl.cloudfarming.client.isobus.model.IsobusLineString;
import nl.cloudfarming.client.isobus.model.IsobusPartField;
import nl.cloudfarming.client.isobus.model.IsobusPoint;
import nl.cloudfarming.client.isobus.model.IsobusPolygon;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;

/**
 * Creates Children for Partfield-nodes.
 * 
 * Partfields can have a collection of Polygons, LineStrings or Points.
 * 
 * @author Gerben Feenstra
 */
@Messages({
    "isobus.project.taskdata.node.name.linestring_partfield=LineString ({0})",
    "isobus.project.taskdata.node.name.point_partfield=Point ({0}, {1})",
    "isobus.project.taskdata.node.name.polygon_partfield=Polygon ({0})"
})
public class PartFieldChildFactory extends ChildFactory<Node> {

    private static final String LINESTRING_NODE_ICON = "";
    private static final String POLYGON_NODE_ICON = "";
    private static final String POINT_NODE_ICON = "";
    
    private final IsobusPartField partField;

    public PartFieldChildFactory(IsobusPartField partField) {
        this.partField = partField;
    }

    @Override
    protected boolean createKeys(List<Node> toPopulate) {

        // LineString (LSG)
        for (IsobusLineString lineString : partField.getLSG()) {
            String displayName = NbBundle.getMessage(PartFieldChildFactory.class, "isobus.project.taskdata.node.name.linestring_partfield", lineString.getName());
            toPopulate.add(new IsobusCommonNode(Children.create(new LineStringChildFactory(lineString), true), lineString, LINESTRING_NODE_ICON, displayName));
        }

        // Polygon (PLN)
        for (IsobusPolygon polygon : partField.getPLN()) {
            String displayName = NbBundle.getMessage(PartFieldChildFactory.class, "isobus.project.taskdata.node.name.polygon_partfield", polygon.getPolygonDesignator());
            toPopulate.add(new IsobusCommonNode(Children.create(new PolygonChildFactory(polygon), true), polygon, POLYGON_NODE_ICON, displayName));
        }
        
        // Point (PNT)
        for (IsobusPoint point : partField.getPNT()) {
             String displayName = NbBundle.getMessage(PartFieldChildFactory.class, "isobus.project.taskdata.node.name.point_partfield", point.getPositionEast(), point.getPositionNorth());
            toPopulate.add(new IsobusCommonNode(Children.LEAF, point, POINT_NODE_ICON, displayName));
        }              
        return true;
    }

    @Override
    protected Node createNodeForKey(Node node) {
        return node;
    }
}
