/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.type;

import java.util.List;
import nl.cloudfarming.client.isobus.model.*;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;

/**
 *
 * @author Gerben Feenstra
 */
@Messages({
    "isobus.project.taskdata.aggregatenode.name.commentallocation=Comment allocations",
    "isobus.project.taskdata.aggregatenode.name.connection=Connections",
    "isobus.project.taskdata.aggregatenode.name.deviceallocation=Device allocations",
    "isobus.project.taskdata.aggregatenode.name.datalog=Datalogs",
    "isobus.project.taskdata.aggregatenode.name.productallocation=Product allocations",
    "isobus.project.taskdata.aggregatenode.name.treatmentzone=Treatment zone",
    "isobus.project.taskdata.node.name.grid=Grid ({0})",
    "isobus.project.taskdata.node.name.opertechpractice=OperationTechniquePractice",
    "isobus.project.taskdata.node.name.timelog=TimeLog ({0})",
    "isobus.project.taskdata.node.name.workerallocation=WorkerAllocation"
})
public class TaskChildFactory extends ChildFactory<Node> {

    // Childnode icons
    private static final String GRID_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_grid.png";
    private static final String OPERTECHPRACTICE_CHILDNODE_ICON = "";
    private static final String TIMELOG_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_timelog.png";
    private static final String WORKERALLOCATION_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_workerallocation.png";
    // Aggregate node icons
    private static final String COMMENTALLOCATION_AGGREGATENODE_ICON = "";
    private static final String CONNECTION_AGGREGATENODE_ICON = "";
    private static final String DEVICEALLOCATION_AGGREGATENODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_device-allocation.png";
    private static final String DATALOG_AGGREGATENODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_datalog.png";
    private static final String PRODUCTALLOCATION_AGGREGATENODE_ICON = "";
    private static final String TIME_AGGREGATENODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_time.png";
    private static final String TREATMENTZONE_AGGREGATENODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_treatmentzone.png";
    // Aggregate node displaynames
    private static final String COMMENTALLOCATION_AGGREGATENODE_NAME = NbBundle.getMessage(TaskChildFactory.class, "isobus.project.taskdata.aggregatenode.name.commentallocation");
    private static final String CONNECTION_AGGREGATENODE_NAME = NbBundle.getMessage(TaskChildFactory.class, "isobus.project.taskdata.aggregatenode.name.connection");
    private static final String DEVICEALLOCATION_AGGREGATENODE_NAME = NbBundle.getMessage(TaskChildFactory.class, "isobus.project.taskdata.aggregatenode.name.deviceallocation");
    private static final String DATALOG_AGGREGATENODE_NAME = NbBundle.getMessage(TaskChildFactory.class, "isobus.project.taskdata.aggregatenode.name.datalog");
    private static final String PRODUCTALLOCATION_AGGREGATENODE_NAME = NbBundle.getMessage(TaskChildFactory.class, "isobus.project.taskdata.aggregatenode.name.productallocation");
    private static final String TREATMENTZONE_AGGREGATENODE_NAME = NbBundle.getMessage(TaskChildFactory.class, "isobus.project.taskdata.aggregatenode.name.treatmentzone");
    private final IsobusTask isobusTask;

    public TaskChildFactory(IsobusTask isobusTask) {
        this.isobusTask = isobusTask;
    }

    @Override
    protected boolean createKeys(List<Node> toPopulate) {

        // CommentAllocation (CAN)
        Children commentAllocationChildren = Children.create(new AggregateNodeChildFactory<>(isobusTask.getCAN()), true);
        toPopulate.add(new IsobusAggregateNode(commentAllocationChildren, COMMENTALLOCATION_AGGREGATENODE_ICON, COMMENTALLOCATION_AGGREGATENODE_NAME));

        // Connection (CNN)
        Children connectionAllocationChildren = Children.create(new AggregateNodeChildFactory<>(isobusTask.getCNN()), true);
        toPopulate.add(new IsobusAggregateNode(connectionAllocationChildren, CONNECTION_AGGREGATENODE_ICON, CONNECTION_AGGREGATENODE_NAME));

        // DeviceAllocation (DAN)
        Children deviceAllocationChildren = Children.create(new AggregateNodeChildFactory<>(isobusTask.getDAN()), true);
        toPopulate.add(new IsobusAggregateNode(deviceAllocationChildren, DEVICEALLOCATION_AGGREGATENODE_ICON, DEVICEALLOCATION_AGGREGATENODE_NAME));

        // DataLog (DLT)
        Children dataLogAllocationChildren = Children.create(new AggregateNodeChildFactory<>(isobusTask.getDLT()), true);
        toPopulate.add(new IsobusAggregateNode(dataLogAllocationChildren, DATALOG_AGGREGATENODE_ICON, DATALOG_AGGREGATENODE_NAME));

        // Grid (GRD)
        Grid grid = isobusTask.getGRD();
        if (grid != null) {
            Lookup.getDefault().lookup(Object.class);
            String displayName = NbBundle.getMessage(TaskChildFactory.class, "isobus.project.taskdata.node.name.grid", grid.getFilename());
            toPopulate.add(new IsobusCommonNode(Children.LEAF, grid, GRID_CHILDNODE_ICON, displayName));
        }

        // OperTechPractice (OTP)
        OperationTechniquePractice operTechPractice = isobusTask.getOTP();
        if (operTechPractice != null) {
            String displayName = NbBundle.getMessage(TaskChildFactory.class, "isobus.project.taskdata.node.name.opertechpractice");
            toPopulate.add(new IsobusCommonNode(Children.LEAF, operTechPractice, OPERTECHPRACTICE_CHILDNODE_ICON, displayName));
        }

        // ProductAllocation (PAN)   
        Children productAllocationChildren = Children.create(new AggregateNodeChildFactory<>(isobusTask.getPAN()), true);
        toPopulate.add(new IsobusAggregateNode(productAllocationChildren, PRODUCTALLOCATION_AGGREGATENODE_ICON, PRODUCTALLOCATION_AGGREGATENODE_NAME));

        // TimeLog (TLG) & Time (TIM)
        // There is only 1 timelog (despite it being a list)
        // TODO: Investigate; Wrong assumption? There can be more then 1 timelog in one isobustaskdata, but can there be more then on in a single isobus task?
        if (isobusTask.getTLG() != null && !isobusTask.getTLG().isEmpty()) {
            TimeLog timeLog = isobusTask.getTLG().get(0);
            Children timeChildren = Children.create(new AggregateNodeChildFactory<>(isobusTask.getTIM()), true);
            String timeLogDisplayName = NbBundle.getMessage(TaskChildFactory.class, "isobus.project.taskdata.node.name.timelog", timeLog.getFilename());
            toPopulate.add(new IsobusAggregateNode(timeChildren, TIME_AGGREGATENODE_ICON, timeLogDisplayName));
        }
        // TreatmentZone (TZN)
        Children treatmentZoneChildren = Children.create(new AggregateNodeChildFactory<>(isobusTask.getTZN()), true);
        toPopulate.add(new IsobusAggregateNode(treatmentZoneChildren, TREATMENTZONE_AGGREGATENODE_ICON, TREATMENTZONE_AGGREGATENODE_NAME));

        // WorkerAllocation (WAN)
        for (WorkerAllocation workerAllocation : isobusTask.getWAN()) {
            String displayName = NbBundle.getMessage(TaskChildFactory.class, "isobus.project.taskdata.node.name.workerallocation");
            toPopulate.add(new IsobusCommonNode(Children.create(new WorkerAllocationChildFactory(workerAllocation), true), workerAllocation, WORKERALLOCATION_CHILDNODE_ICON, displayName));
        }
        return true;
    }

    @Override
    protected Node createNodeForKey(Node key) {
        return key;
    }
}
