/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.type;

import java.util.List;
import nl.cloudfarming.client.isobus.model.Comment;
import nl.cloudfarming.client.isobus.model.IsobusColor;
import nl.cloudfarming.client.isobus.model.IsobusTaskData;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;

/**
 *
 * @author Gerben Feenstra
 */
@NbBundle.Messages({
    "isobus.project.taskdata.aggregatenode.name.croptype=Croptypes",
    "isobus.project.taskdata.aggregatenode.name.culturalpractice=Cultural Practices",
    "isobus.project.taskdata.aggregatenode.name.device=Devices",
    "isobus.project.taskdata.aggregatenode.name.farm=Farms",
    "isobus.project.taskdata.aggregatenode.name.partfield=Partfields",
    "isobus.project.taskdata.aggregatenode.name.product=Products",
    "isobus.project.taskdata.aggregatenode.name.productgroup=Product groups",
    "isobus.project.taskdata.aggregatenode.name.valuepresentation=ValuePresentations",
    "isobus.project.taskdata.aggregatenode.name.worker=Workers",  
    "isobus.project.taskdata.aggregatenode.name.task=Tasks",
    "isobus.project.taskdata.node.name.codedcomment=CodedComment ({0} - {1})",
    "isobus.project.taskdata.node.name.color=Color ({0})"
    
})
public class TaskDataChildFactory extends ChildFactory<Node> {

    // Childnode icons
    private static final String COLOR_NODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_color.png";
    private static final String CODEDCOMMENT_NODE_ICON = "";
    
    // Aggregate Node icons
    private static final String VALUEPRESENTATION_AGGREGATENODE_ICON = "";
    private static final String PARTFIELD_AGGREGATENODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_partfield.png";
    private static final String CROPTYPE_AGGREGATENODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_croptype.png";
    private static final String WORKER_AGGREGATENODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_worker.png";
    private static final String PRODUCT_AGGREGATENODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_product.png";
    private static final String PRODUCTGROUP_AGGREGATENODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_productgroup.png";
    private static final String CULTURALPRACTICE_AGGREGATENODE_ICON = "";
    private static final String DEVICE_AGGREGATENODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_device.png";
    private static final String TASK_AGGREGATENODE_ICON = "";
    private static final String FARM_AGGREGATENODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_farm.png";
    private final IsobusTaskData isobusObj;

    public TaskDataChildFactory(IsobusTaskData isobusObj) {
        this.isobusObj = isobusObj;
    }

    @Override
    protected boolean createKeys(List<Node> toPopulate) {

        // XML Main node
        IsobusTaskData isobusTaskData = (IsobusTaskData) isobusObj;

        // Color (CLD)
        for (IsobusColor color : isobusTaskData.getCLD()) {
            String displayName = NbBundle.getMessage(TaskDataChildFactory.class, "isobus.project.taskdata.node.name.color", color.getId());
            toPopulate.add(new IsobusCommonNode(Children.create(new ColorChildFactory(color), true), color, COLOR_NODE_ICON, displayName));
        }

        // ValuePresentation (VPN)
        Children valuePresentationChildren = Children.create(new AggregateNodeChildFactory<>(isobusTaskData.getVPN()), true);
        toPopulate.add(new IsobusAggregateNode(valuePresentationChildren, VALUEPRESENTATION_AGGREGATENODE_ICON, NbBundle.getMessage(TaskDataChildFactory.class, "isobus.project.taskdata.aggregatenode.name.valuepresentation")));

        // Farm (FRM)
        Children farmChildren = Children.create(new AggregateNodeChildFactory<>(isobusTaskData.getFRM()), true);
        toPopulate.add(new IsobusAggregateNode(farmChildren, FARM_AGGREGATENODE_ICON, NbBundle.getMessage(TaskDataChildFactory.class, "isobus.project.taskdata.aggregatenode.name.farm")));

        // Partfield (PFD)
        //Children partFieldChildren = Children.create(new AggregateNodeChildFactory<IsobusPartField>(isobusTaskData.getPFD()), true);
        Children partFieldChildren = Children.create(new PartFieldNodeFactory(isobusTaskData.getPFD()), true);
        toPopulate.add(new IsobusAggregateNode(partFieldChildren, PARTFIELD_AGGREGATENODE_ICON, NbBundle.getMessage(TaskDataChildFactory.class, "isobus.project.taskdata.aggregatenode.name.partfield")));

        // Croptype (CTP)
        Children cropTypeChildren = Children.create(new AggregateNodeChildFactory<>(isobusTaskData.getCTP()), true);
        toPopulate.add(new IsobusAggregateNode(cropTypeChildren, CROPTYPE_AGGREGATENODE_ICON, NbBundle.getMessage(TaskDataChildFactory.class, "isobus.project.taskdata.aggregatenode.name.croptype")));

        // Worker (WKR)
        Children workerChildren = Children.create(new AggregateNodeChildFactory<>(isobusTaskData.getWKR()), true);
        toPopulate.add(new IsobusAggregateNode(workerChildren, WORKER_AGGREGATENODE_ICON, NbBundle.getMessage(TaskDataChildFactory.class, "isobus.project.taskdata.aggregatenode.name.worker")));

        // ProductGroup (PGP)
        Children productGroupChildren = Children.create(new AggregateNodeChildFactory<>(isobusTaskData.getPGP()), true);
        toPopulate.add(new IsobusAggregateNode(productGroupChildren, PRODUCTGROUP_AGGREGATENODE_ICON, NbBundle.getMessage(TaskDataChildFactory.class, "isobus.project.taskdata.aggregatenode.name.productgroup")));

        // Product (PDT)
        Children productChildren = Children.create(new AggregateNodeChildFactory<>(isobusTaskData.getPDT()), true);
        toPopulate.add(new IsobusAggregateNode(productChildren, PRODUCT_AGGREGATENODE_ICON, NbBundle.getMessage(TaskDataChildFactory.class, "isobus.project.taskdata.aggregatenode.name.product")));

        // CulturalPractice (CPC)
        Children culturalPracticeChildren = Children.create(new AggregateNodeChildFactory<>(isobusTaskData.getCPC()), true);
        toPopulate.add(new IsobusAggregateNode(culturalPracticeChildren, CULTURALPRACTICE_AGGREGATENODE_ICON, NbBundle.getMessage(TaskDataChildFactory.class, "isobus.project.taskdata.aggregatenode.name.culturalpractice")));

        // Device (DVC)"isobus.project.taskdata.aggregatenode.name.product=Products",
        Children deviceChildren = Children.create(new AggregateNodeChildFactory<>(isobusTaskData.getDVC()), true);
        toPopulate.add(new IsobusAggregateNode(deviceChildren, DEVICE_AGGREGATENODE_ICON, NbBundle.getMessage(TaskDataChildFactory.class, "isobus.project.taskdata.aggregatenode.name.device")));


        // CodedComment (CCT)
        for (Comment codedComment : isobusTaskData.getCCT()) {
            String displayName = NbBundle.getMessage(TaskDataChildFactory.class, "isobus.project.taskdata.node.name.codedcomment", codedComment.getId(), codedComment.getDesignator());
            toPopulate.add(new IsobusCommonNode(Children.create(new CommentChildFactory(codedComment), true), codedComment, CODEDCOMMENT_NODE_ICON, displayName));
        }

        // Task (TSK)
        Children taskChildren = Children.create(new AggregateTaskChildFactory(isobusTaskData.getTSK()), true);
        toPopulate.add(new IsobusAggregateNode(taskChildren, TASK_AGGREGATENODE_ICON, NbBundle.getMessage(TaskDataChildFactory.class, "isobus.project.taskdata.aggregatenode.name.task")));   

        
        
//        for (IsobusTask task : isobusTaskData.getTSK()) {
//            String displayName = NbBundle.getMessage(TaskDataChildFactory.class, "isobus.project.taskdata.node.name.task", task.getTaskId(), task.getTaskDesignator());
//            toPopulate.add(new IsobusCommonNode(Children.create(new TaskChildFactory(task), true), task, TASK_NODE_ICON, displayName));
//        }
        
        return true;
    }

    @Override
    protected Node createNodeForKey(Node key) {
        return key;
    }
}
